package com.persagy.dc.digital.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.dc.basic.utils.DigitalMessageHelper;
import com.persagy.dc.common.constant.ValidEnum;
import com.persagy.dc.common.model.entity.BaseEntity;
import com.persagy.dc.digital.dao.ObjectDigitalMapper;
import com.persagy.dc.digital.entity.ObjectDigital;
import com.persagy.dc.digital.service.IObjectDigitalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 对象数据 实现类
 * @author Charlie Yu
 * @date 2021-07-08
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ObjectDigitalServiceImpl implements IObjectDigitalService {

    @Resource
    private ObjectDigitalMapper dao;
    @Autowired
    private DigitalMessageHelper messageSender;

    @Override
    public ObjectDigital load(String id) {
        return dao.selectById(id);
    }

    @Override
    public List<ObjectDigital> queryByCondition(Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectList(queryWrapper);
    }

    @Override
    public Page queryByCondition(Page page, Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectPage(page, queryWrapper);
    }

    @Override
    public List<ObjectDigital> insert(List<ObjectDigital> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
//        validateOnSave(voList, false);
        voList.forEach(vo -> {
            // 新增
            dao.insert(vo);
        });
        return voList;
    }

    @Override
    public List<ObjectDigital> update(List<ObjectDigital> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
//        validateOnSave(voList, true);
        List<String> idList = CollUtil.getFieldValues(voList, BaseEntity.PROP_ID, String.class);
        List<ObjectDigital> dbList = dao.selectBatchIds(idList);
        Map<String, ObjectDigital> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        voList.forEach(vo -> {
            ObjectDigital dbVO = MapUtil.get(dbMap, vo.getId(), ObjectDigital.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 修改
            dao.updateById(vo);
        });
        return voList;
    }

    @Override
    public void delete(List<String> idList) {
        if(CollUtil.isEmpty(idList)) {
            return;
        }
        List<ObjectDigital> dbList = dao.selectBatchIds(idList);
        Map<String, ObjectDigital> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        idList.forEach(id -> {
            ObjectDigital dbVO = MapUtil.get(dbMap, id, ObjectDigital.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 删除
            ObjectDigital vo = new ObjectDigital();
            vo.setId(id);
            vo.setValid(ValidEnum.FALSE.getType());
            dao.updateById(vo);
        });
    }
}
