package com.persagy.dc.parser.entity;

import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dc.define.entity.ObjectInfo;
import lombok.Data;

/**
 * 信息点 处理结果
 * @author Charlie Yu
 * @date 2021-07-12
 */
@Data
public class ParserResult {
    /** 信息点定义 */
    private ObjectInfo infoDefine;
    /** 信息点值 */
    private JsonNode valueInfo;
    /** 错误信息 */
    private StringBuilder errorMsg = new StringBuilder();

    public ParserResult(ObjectInfo infoDefine, JsonNode valueInfo) {
        this.infoDefine = infoDefine;
        this.valueInfo = valueInfo;
    }

    /**
     * 是否复选
     * @return
     */
    public boolean isMulti() {
        return BooleanUtil.isTrue(infoDefine.getMultipleFlag());
    }

    /**
     * 是否区间
     * @return
     */
    public boolean isRegion() {
        return BooleanUtil.isTrue(infoDefine.getRegionFlag());
    }

    /**
     * 追加错误信息
     * @param error 错误信息
     */
    public void addErrorMsg(String error) {
        errorMsg.append(error);
    }

    /**
     * 追加错误信息
     * @param error 错误信息
     * @param params 格式信息
     */
    public void addErrorMsg(String error, Object... params) {
        errorMsg.append(StrUtil.format(error, params));
    }
}
