package com.persagy.dc.parser.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.persagy.dc.parser.entity.EnumVO;
import com.persagy.dc.parser.entity.ParserResult;

import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 枚举信息点 处理器
 * @author Charlie Yu
 * @date 2021-07-12
 */
public class EnumInfoParser extends AbstractInfoDataParser {

    @Override
    protected boolean validateDefine(ParserResult result) {
        // 字符串类型不支持区间格式
        if(result.isRegion()) {
            result.addErrorMsg("信息点[{}]定义有误，ENUM类型不允许定义区间！", result.getInfoDefine().getCode());
            return false;
        }
        // 数据源 - 应为区间列表格式
        try {
            formatListValue(result.getInfoDefine().getDataSource(), EnumVO.class);
        } catch (IOException e) {
            result.addErrorMsg("信息点[{}]定义有误，数据源格式有误！", result.getInfoDefine().getCode());
            return false;
        }
        return true;
    }

    @Override
    protected boolean validateValue(ParserResult result) {
        try {
            // 数据源定义信息
            List<EnumVO> dataSource = formatListValue(result.getInfoDefine().getDataSource(), EnumVO.class);
            Set<String> codeSet = new HashSet<>();
            for(EnumVO enumVO:dataSource) {
                codeSet.add(enumVO.getCode());
            }
            // 复数
            if(result.isMulti()) {
                List<String> valueList = formatListValue(result.getValueInfo(), String.class);
                if(!CollUtil.containsAll(codeSet, valueList)) {
                    result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                    return false;
                }
            } else {
                // 单数
                String value = formatValue(result.getValueInfo(), String.class);
                if(!codeSet.contains(value)) {
                    result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                    return false;
                }
            }
        } catch (IOException e) {
            result.addErrorMsg("信息点[{}]数据格式有误！", result.getInfoDefine().getCode());
            return false;
        }
        return true;
    }
}
