package com.persagy.dc.parser.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.persagy.dc.common.model.data.RegionValue;
import com.persagy.dc.common.utils.RegionUtil;
import com.persagy.dc.parser.entity.ParserResult;

import java.io.IOException;
import java.util.List;

/**
 * 数值信息点 处理器
 * @author Charlie Yu
 * @date 2021-07-12
 */
public class NumberInfoParser <T extends Comparable> extends AbstractInfoDataParser {

    @Override
    protected boolean validateDefine(ParserResult result) {
        // 数据源 - 应为区间列表格式
        try {
            formatRegionListValue(result.getInfoDefine().getDataSource(), getClassType());
        } catch (IOException e) {
            result.addErrorMsg("信息点[{}]定义有误，数据源格式有误！", result.getInfoDefine().getCode());
            return false;
        }
        return true;
    }

    @Override
    protected boolean validateValue(ParserResult result) {
        try {
            // 数据源定义信息
            List<RegionValue<T>> dataSource = formatRegionListValue(result.getInfoDefine().getDataSource(), getClassType());
            // 复数
            if(result.isMulti()) {
                // 复数区间
                if(result.isRegion()) {
                    List<RegionValue<T>> valueList = formatRegionListValue(result.getValueInfo(), getClassType());
                    // 区间范围校验
                    if (CollUtil.isNotEmpty(dataSource)) {
                        if (!RegionUtil.allContains(dataSource, valueList)) {
                            result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                            return false;
                        }
                    }
                } else {
                    // 复数非区间
                    List<T> valueList = formatListValue(result.getValueInfo(), getClassType());
                    // 区间范围校验
                    if (CollUtil.isNotEmpty(dataSource)) {
                        if (!RegionUtil.allContain(dataSource, valueList)) {
                            result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                            return false;
                        }
                    }
                }
            } else {
                // 单数区间
                if(result.isRegion()) {
                    RegionValue<T> value = formatRegionValue(result.getValueInfo(), getClassType());
                    if (CollUtil.isNotEmpty(dataSource)) {
                        // 区间范围校验
                        if (!RegionUtil.anyContains(dataSource, value)) {
                            result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                            return false;
                        }
                    }
                } else {
                    // 单数非区间
                    T value = formatValue(result.getValueInfo(), getClassType());
                    if (CollUtil.isNotEmpty(dataSource)) {
                        // 区间范围校验
                        if (!RegionUtil.anyContain(dataSource, value)) {
                            result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                            return false;
                        }
                    }
                }
            }
        } catch (IOException e) {
            result.addErrorMsg("信息点[{}]数据格式有误！", result.getInfoDefine().getCode());
            return false;
        }
        return true;
    }

    /**
     * 泛型类
     * @return
     */
    protected Class<T> getClassType() {
        return null;
    }
}
