package com.persagy.dc.parser.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.BooleanUtil;
import com.persagy.dc.common.model.data.RegionValue;
import com.persagy.dc.common.utils.RegionUtil;
import com.persagy.dc.parser.entity.ParserResult;

import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 字符串信息点 处理器
 * @author Charlie Yu
 * @date 2021-07-12
 */
public class StringInfoParser extends AbstractInfoDataParser{

    @Override
    protected boolean validateDefine(ParserResult result) {
        boolean regionFlag = BooleanUtil.isTrue(result.getInfoDefine().getRegionFlag());
        // 字符串类型不支持区间格式
        if(regionFlag) {
            result.addErrorMsg("信息点[{}]定义有误，STRING类型不允许定义区间！", result.getInfoDefine().getCode());
            return false;
        }
        // 数据源 - 应为区间列表格式
        try {
            formatRegionListValue(result.getInfoDefine().getDataSource(), Integer.class);
        } catch (IOException e) {
            result.addErrorMsg("信息点[{}]定义有误，数据源格式有误！", result.getInfoDefine().getCode());
            return false;
        }
        return true;
    }

    @Override
    protected boolean validateValue(ParserResult result) {
        boolean multiFlag = BooleanUtil.isTrue(result.getInfoDefine().getMultipleFlag());
        try {
            // 数据源定义信息
            List<RegionValue<Integer>> dataSource = formatRegionListValue(result.getInfoDefine().getDataSource(), Integer.class);
            // 复数
            if(multiFlag) {
                List<String> valueList = formatListValue(result.getValueInfo(), String.class);
                if(CollUtil.isNotEmpty(dataSource)) {
                    // 字符串长度校验
                    List<Integer> lengthList = valueList.stream().map(String::length).collect(Collectors.toList());
                    if(!RegionUtil.allContain(dataSource, lengthList)) {
                        result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                        return false;
                    }
                }
            } else {
                // 单数
                String value = formatValue(result.getValueInfo(), String.class);
                if(CollUtil.isNotEmpty(dataSource)) {
                    // 字符串长度校验
                    if(!RegionUtil.anyContain(dataSource, value.length())) {
                        result.addErrorMsg("信息点[{}]数据有误，超出区间范围！", result.getInfoDefine().getCode());
                        return false;
                    }
                }
            }
        } catch (IOException e) {
            result.addErrorMsg("信息点[{}]数据格式有误！", result.getInfoDefine().getCode());
            return false;
        }
        return true;
    }

}
